#!/bin/sh
#
# SyncPro startup script for Unix/Linux
#
clear
# Setting up running class
MAIN_CLASS=Main
# Setting up memory usage for SyncPro
JVM_ARGS="-Xms256m -Xmx1024m"

# Reference : http://agiletrack.net/samples/sample-detect-java.html
# This script attempts to find an existing installation of Java that meets a minimum version
# requirement on a Linux machine.  If it is successful, it will export a JAVA_HOME environment
# variable that can be used by another calling script.
#
# To specify the required version, set the REQUIRED_VERSION to the major version required, 
# e.g. 1.3, but not 1.3.1.
REQUIRED_VERSION=1.6

# Transform the required version string into a number that can be used in comparisons
REQUIRED_VERSION=`echo $REQUIRED_VERSION | sed -e 's;\.;0;g'`
# Check JAVA_HOME directory to see if Java version is adequate
JAVA_HOME=/usr
if [ $JAVA_HOME ]
then
	JAVA_EXE=$JAVA_HOME/bin/java
	$JAVA_EXE -version 2> tmp.ver
	VERSION=`cat tmp.ver | grep "java version" | awk '{ print substr($3, 2, length($3)-2); }'`
	rm tmp.ver
	VERSION=`echo $VERSION | awk '{ print substr($1, 1, 3); }' | sed -e 's;\.;0;g'`
	if [ $VERSION ]
	then
		if [ $VERSION -ge $REQUIRED_VERSION ]
		then
			JAVA_HOME=`echo $JAVA_EXE | awk '{ print substr($1, 1, length($1)-9); }'`
		else
			JAVA_HOME=
		fi
	else
		JAVA_HOME=
	fi
fi

# If the existing JAVA_HOME directory is adequate, then leave it alone
# otherwise, use 'locate' to search for other possible java candidates and
# check their versions.
if [ $JAVA_HOME ]
then
	:
else
	for JAVA_EXE in `locate bin/java | grep java$ | xargs echo`
	do
		if [ $JAVA_HOME ] 
		then
			:
		else
			$JAVA_EXE -version 2> tmp.ver 1> /dev/null
			VERSION=`cat tmp.ver | grep "java version" | awk '{ print substr($3, 2, length($3)-2); }'`
			rm tmp.ver
			VERSION=`echo $VERSION | awk '{ print substr($1, 1, 3); }' | sed -e 's;\.;0;g'`
			if [ $VERSION ]
			then
				if [ $VERSION -ge $REQUIRED_VERSION ]
				then
					JAVA_HOME=`echo $JAVA_EXE | awk '{ print substr($1, 1, length($1)-9); }'`
				fi
			fi
		fi
	done
fi

# If the correct Java version is detected, then export the JAVA_HOME environment variable
if [ $JAVA_HOME ]
then
	export JAVA_HOME
	echo Java Home Path : $JAVA_HOME
	$JAVA_EXE -version 2> tmp.ver 1> /dev/null
	VERSION=`cat tmp.ver | grep "java version" | awk '{ print substr($3, 2, length($3)-2); }'`
	rm tmp.ver
	echo Current Java Runtime Environment Version : $VERSION
	
	# Check existence of java home path and setup executable java
	JAVA_EXECUTABLE=$JAVA_HOME/bin/java
	if [ -f "$JAVA_EXECUTABLE" ]
	then
		# Setting home path execution of SyncPro
		HOME=.
		# Setting Current Path
		CLASSPATH=$HOME"/SyncPro.jar"
		# Setting my doggy 1.4.2 class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/mydoggy-1.4.2/mydoggy-api-1.4.2.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/mydoggy-1.4.2/mydoggy-plaf-1.4.2.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/mydoggy-1.4.2/mydoggy-res-1.4.2.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/mydoggy-1.4.2/TableLayout-20050920.jar"
		# Setting dom4j 1.6.1 class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/dom4j-1.6.1/dom4j-1.6.1.jar"
		# Setting gdswing class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/gdswing/gdswing.jar"
		# Setting jfreeChart 1.0.13 class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/jfreechart/jcommon-1.0.16.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/jfreechart/jfreechart-1.0.13.jar"
		# Setting jrap class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/Jrap/jakarta-regexp-1.4.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/Jrap/jrap_3.0.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/Jrap/jrap-dist.jar"
		# Setting xerces class path (Run time lib for Jrap)
		CLASSPATH=$CLASSPATH":"$HOME"/lib/xerces/resolver.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/xerces/xercesImpl.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/xerces/xercesSamples.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/xerces/xml-apis.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/xerces/xmlParserAPIs.jar"
		# Setting Log4j class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/apache-log4j-1.2.15/log4j-1.2.15.jar"
		# Setting Apache commoms math 2.0 class path
		CLASSPATH=$CLASSPATH":"$HOME"/lib/apache-commons-math-2.0/commons-math-2.0.jar"
		#Setting Apache commons configuration 1.6 class path and its dependencies
		CLASSPATH=$CLASSPATH":"$HOME"/lib/apache-commons-configuration-1.6/commons-configuration-1.6.jar"
		CLASSPATH=$CLASSPATH":"$HOME"/lib/apache-commons-configuration-1.6/commons-lang-2.5.jar"
		# Running SyncPro program
		echo $JAVA_EXECUTABLE -cp $CLASSPATH $JVM_ARGS $MAIN_CLASS 
		$JAVA_EXECUTABLE -cp $CLASSPATH $JVM_ARGS $MAIN_CLASS 
	else
		# Encountering any errors in the java executable file
		echo "Please check the path if Java Executable exists in the path"
		echo "You may get the latest version of Java at http://java.sun.com/javase/downloads/index.jsp"
		read -n 1 -p "Press any key to continue..." INP
		if [[ $INP != '' ]] ; then
				echo -ne '\b \n'
		fi
	fi
else
	# Encountering any errors in the java executable file
	echo "Please check if Java Runtime Environment (Version 1.6) is installed in your Unix/Linux system."
	echo "You may get the latest version of Java at http://java.sun.com/javase/downloads/index.jsp"
	read -n 1 -p "Press any key to continue..." INP
	if [[ $INP != '' ]] ; then
			echo -ne '\b \n'
	fi
fi
